; LED4d.ASM
;
; flash sequence 5 LEDs on GP0, GP1, GP2, GP4, and GP5 using stored pattern
;  and reverse direction based on button push on GP3


	list      p=12f629            		; list directive to define processor
	#include <p12f629.inc>        		; processor specific variable definitions

	errorlevel  -302              		; suppress message 302 from list file

	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT  

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.

;***** VARIABLE DEFINITIONS

pointer		EQU		0x20		;next position in pattern
flags		EQU		0x21		;bit flags 0=has timer0 interrupt delay happened?
check		EQU		0x22
spos		EQU		0x23		; start poistion for pattern	
epos		EQU		0x24		; end position for pointer
adder		EQU		0x25		;what to add to pointer
savedW		EQU		0x28		;preserve W on interrupt
loop1		EQU		0x30		;tmr0 delay counters
loop2		EQU		0x31
cnt1		EQU		0x32		;tmr0 delays
cnt2		EQU		0x33


;***** CONSTANTS

lp1			EQU		4			;initial timer0 delays
lp2			EQU		4

interrupt	EQU		0
btnPrssd	EQU		1
btnProcd	EQU		2
atstart		EQU		3
atend		EQU		4


;***** MACROS

bank0:  MACRO 
        bcf STATUS, RP0 
        ENDM 

bank1:  MACRO 
        bsf STATUS, RP0 
        ENDM 


;***** PROGRAM CODE

		ORG     0x000					; processor reset vector

		goto    Main					; go to beginning of program


		ORG     0x004					; interrupt vector location

Interrupt
		movwf	savedW					; save & preserve W register

		btfss	INTCON, T0IF			; only concerned with TMR0 interrupts
		goto	IntDone

		bcf		INTCON, T0IF

		btfss	GPIO, GP3				; is button pressed?
		goto	Btn1					;  - no

		btfss	flags, btnPrssd				; has button already been pressed?
		goto	Btn2					;  - no

		btfsc	flags, btnProcd			; has button press been processed?
		goto	Btn3					;  - yes

		bsf		flags, btnProcd

		movlw	b'11111110'
		xorwf	adder, F				; change dir

		goto	Btn3

Btn2
		bsf		flags, btnPrssd			; set button has already been pressed
		bcf		flags, btnProcd			; clear button press processed flag
		goto	Btn3

Btn1
		bcf		flags, btnPrssd			; clear button pressed flag

Btn3
		decfsz	loop1, F
		goto	IntDone

		movf	cnt1, W
		movwf	loop1

		decfsz	loop2, F
		goto	IntDone
		
		movf	cnt2, W
		movwf	loop2

		bsf		flags, interrupt		; mark delay has occurred

IntDone
		movf	savedW, W
		retfie							; return from interrupt

; ********

Main
		movlw	lp1						; initialise interrupt variables
		movwf	loop1					;
		movwf	cnt1					;
		movlw	lp2						;
		movwf	loop2					;
		movwf	cnt2					;

		movlw	1						; initial adder direction +1
		movwf	adder

		bank1
		
		movlw	b'001000'				; set TRISIO, all output except GP3
		movwf	TRISIO

		movlw	b'10000101'				; disable internal pull-ups and set pre-scalar
		movwf	OPTION_REG

		movlw	b'10100000'				; use TMR0 and disable all other interrupt sources 
		movwf	INTCON

		bank0

		clrf	pointer
		decf	pointer, F

		movlw	0
		movwf	spos

		movlw	4
		movwf	epos

Loop1
		movf	adder, W
		addwf	pointer, F

		movf	pointer, W
		call	LookUp
		movwf	GPIO

Delay									; wait for interrupt flag
		btfss	flags, interrupt
		goto	Delay
		bcf		flags, interrupt

		btfsc	adder, 7
		goto	CheckS

		bcf		flags, atend
		movf	epos, W
		movwf	check
		movf	pointer, W
		xorwf	check, F
		btfsc	STATUS, Z
		bsf		flags, atend

		movf	pointer, W
		btfsc	flags, atend
		movf	spos, W
		movwf	pointer

		btfsc	flags, atend
		decf	pointer, F

		goto	Loop1

CheckS
		bcf		flags, atstart
		movf	spos, W
		movwf	check
		movf	pointer, W
		xorwf	check, F
		btfsc	STATUS, Z
		bsf		flags, atstart

		movf	pointer, W
		btfsc	flags,atstart
		movf	epos, W
		movwf	pointer

		btfsc	flags, atstart
		incf	pointer, F

		goto	Loop1

LookUp
		addwf	PCL, F

		retlw	b'000001'				; pattern
		retlw	b'000010'
		retlw	b'000100'
		retlw	b'010000'
		retlw	b'100000'


		END                  			; directive 'end of program'
